<?php

namespace Drupal\apitemplate_io\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Test tool for the API client.
 */
class TestToolForm extends FormBase {

  /**
   * The apitemplate_io.client service.
   *
   * @var \Drupal\apitemplate_io\ApiTemplateClient
   */
  protected $client;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->client = $container->get('apitemplate_io.client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'apitemplate_io_test_tool';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [];

    $templates = $this->client->listTemplates();

    if (!$templates) {
      if ($templates === FALSE) {
        $this->messenger()->addError($this->t('Failed to get the list of templates from APITemplate.io; please, verify the credentials in the Settings page.'));
      }
      elseif (empty($templates)) {
        $this->messenger()->addError($this->t('No templates found; please add at least 1 template in APITemplate.io dashboard.'));
      }

      return $form;
    }

    $form['template'] = [
      '#type' => 'select',
      '#title' => $this->t('Template'),
      '#options' => array_column($templates, 'name', 'template_id'),
      '#description' => $this->t('Template to use for rendering.'),
      '#required' => TRUE,
    ];

    $form['variables'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Variables (JSON)'),
      '#default_value' => '{}',
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Download PDF'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $vars = json_decode($form_state->getValue('variables', ''), TRUE);

    if ($vars === NULL) {
      $form_state->setErrorByName('variables', $this->t('The provided JSON is not valid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $template_id = $form_state->getValue('template');
    $vars = json_decode($form_state->getValue('variables', ''), TRUE);
    if ($pdf_url = $this->client->createPdf($vars, $template_id)) {
      $form_state->setResponse($this->client->serveExternalFile($pdf_url, 'drupal-apitemplate_io-test.pdf'));
    }
  }

}
